@@ -36,11 +36,4 @@ public interface AttributeCheckingMBean {
     public String getString();
 
     public String getNull();
-
-    public long getBytes();
-
-    public float getLongSeconds();
-
-    public double getSmallMinutes();
-
 }
