@@ -8,7 +8,7 @@ use JMX::Jmx4Perl::Response;
 use JMX::Jmx4Perl::Alias;
 use Data::Dumper;
 use Nagios::Plugin;
-use Time::HiRes qw(gettimeofday);
+use Time::HiRes qw(gettimeofday tv_interval);
 use Carp;
 use Scalar::Util qw(looks_like_number);
 use URI::Escape;
@@ -21,15 +21,15 @@ L<check_jmx4perl>
 =head1 SYNOPSIS
 
   # One line in check_jmx4perl to rule them all
-  JMX::Jmx4Perl::CheckJmx4Perl->new(@ARGV)->execute();
+  JMX::Jmx4Perl::CheckJmx4Perl->new()->execute();
 
 =head1 DESCRIPTION
 
 The purpose of this module is to encapsulate a single run of L<check_jmx4perl> 
 in a perl object. This allows for C<check_jmx4perl> to run within the embedded
-Nagios perl interpreter (ePN) wihout interfering with other, potentiall
+Nagios perl interpreter (ePN) wihout interfering with other, potential
 concurrent, runs of this check. Please refer to L<check_jmx4perl> for
-documentation on how to use this check. This module is probbaly E<not> of 
+documentation on how to use this check. This module is probbaly I<not> of 
 general interest and serves only the purpose described above.
 
 =cut
@@ -71,7 +71,6 @@ sub execute {
         
         my $resp = $self->_send_request($jmx,$request);
         my $value = $resp->value;
-        
         # Delta handling
         my $delta = $o->get("delta");
         if (defined($delta)) {
@@ -89,9 +88,9 @@ sub execute {
         $np->add_perfdata(label => $self->_get_name(),value => $value,
                           critical => $o->critical, warning => $o->warning, 
                           $o->base ? (uom => '%') : ());
-        
-        # Verify thresholds
-        my $code = $np->check_threshold($value);    
+
+        my $code = $self->_check_threshhold($value);
+
         return $np->nagios_exit($code,$self->_get_name(). " : Threshold " . 
                          ($code == CRITICAL ? $o->critical : $o->warning) . 
                          " failed for value $value") if $code != OK;
@@ -132,15 +131,14 @@ sub _send_request {
         if ($o->user) {
             print "Remote User: ",$o->user,"\n";
         }
-        $start_time = (gettimeofday)[1];    
+        $start_time = [gettimeofday];
     }
 
     my $resp = $jmx->request($request);
     $self->_verify_response($resp);
 
     if ($o->verbose) {
-        my $duration = int ((gettimeofday)[1] - $start_time) / 1000;
-        print "Result fetched in ",$duration,"ms:\n";
+        print "Result fetched in ",tv_interval($start_time) * 1000," ms:\n";
         print Dumper($resp);
     }
 
@@ -204,7 +202,8 @@ sub _verify_response {
         $np->nagios_die("Error: ".$resp->status." ".$resp->error_text."\nStacktrace:\n".$resp->stacktrace);
     }
     if (!defined($resp->value)) {
-        $np->nagios_die("JMX Request " . $self->_get_name() . " failed " . Dumper($resp));
+        $np->nagios_die("JMX Request " . $self->_get_name() . " returned a null value which can't be used yet. " . 
+                        "Please let me know, whether you need such check for a null value");
     }
     if (ref($resp->value)) { 
         $np->nagios_die("Response value is a ".ref($resp->value).
@@ -223,11 +222,6 @@ sub _verify_and_initialize {
     $np->nagios_die("At least a critical or warning threshold must be given") 
       if ((!defined($o->critical) && !defined($o->warning)));
     
-    $np->set_thresholds
-      (
-       $o->critical ? (critical => $o->critical) : (),
-       $o->warning ? (warning => $o->warning) : ()
-      );    
 }
 
 sub _delta_value {
@@ -280,6 +274,47 @@ sub _base_value {
     return $resp->value;
 }
 
+sub _check_threshhold {
+    my $self = shift;
+    my $value = shift;
+    my $np = $self->{np};
+    my $o = $self->{opts};
+    my $numeric_check;
+    if ($o->numeric || $o->string) {
+        $numeric_check = $o->numeric ? 1 : 0;
+    } else {
+        $numeric_check = looks_like_number($value);
+    }
+    if ($numeric_check) {
+        # Verify numeric thresholds
+        my @ths = 
+          (
+           $o->critical ? (critical => $o->critical) : (),
+           $o->warning ? (warning => $o->warning) : ()
+          );            
+        return $np->check_threshold(check => $value,@ths);    
+    } else {
+        return
+          $self->_check_string_threshold($value,CRITICAL,$o->critical) ||
+            $self->_check_string_threshold($value,WARNING,$o->warning) ||
+              OK;
+    }
+}
+
+sub _check_string_threshold {
+    my $self = shift;
+    my ($value,$level,$check_value) = @_;
+    return undef unless $check_value;
+    if ($check_value =~ m|^\s*qr(.)(.*)\1\s*$|) {
+        return $value =~ m/$2/ ? $level : undef;
+    }
+    if ($check_value =~ s/^\!//) {
+        return $value ne $check_value ? $level : undef; 
+    } else {
+        return $value eq $check_value ? $level : undef;
+    }    
+}
+
 # =========================================================================================== 
 
 sub _create_nagios_plugin {
@@ -325,7 +360,7 @@ sub _create_nagios_plugin {
                  help => "Operation to execute",
                 );
     $np->add_arg(
-                 spec => "base|base-aliase|b=s",
+                 spec => "base|base-alias|b=s",
                  help => "Base alias name, which when given, interprets critical and warning values as relative in the range 0 .. 100%",
                 );
     $np->add_arg(
@@ -336,12 +371,24 @@ sub _create_nagios_plugin {
                  spec => "path|p=s",
                  help => "Inner path for extracting a single value from a complex attribute or return value (e.g. \"used\")",
                 );
+    $np->add_arg(
+                 spec => "string",
+                 help => "Force string comparison for critical and warning checks"
+                );
+    $np->add_arg(
+                 spec => "numeric",
+                 help => "Force numeric comparison for critical and warning checks"
+                );
     $np->add_arg(
                  spec => "critical|c=s",
                  help => "Critical Threshold for value. " . 
                  "See http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT " .
                  "for the threshold format.",
                 );
+    $np->add_arg(
+                 spec => "warning|w=s",
+                 help => "Warning Threshold for value.",
+                );
     $np->add_arg(
                  spec => "proxy=s",
                  help => "Proxy to use"
@@ -354,10 +401,6 @@ sub _create_nagios_plugin {
                  spec => "password=s",
                  help => "Password for HTTP authentication"
                 );
-    $np->add_arg(
-                 spec => "warning|w=s",
-                 help => "Warning Threshold for value.",
-                );
     $np->add_arg(
                  spec => "name|n=s",
                  help => "Name to use for output. Optional, by default a standard value based on the MBean ".
